-- Copyright (c) 2012 Cloudera, Inc. All rights reserved.
--
-- Version 58 of the CMF schema. Migration code to be applied on top of the
-- previous version.
--
-- This version is a little difficult to follow because Postgres doesn't
-- support variables unless you use PL/pgSQL, which I didn't want to do.
-- Look at the MySQL version to see a clearer version of this migration
-- with variables.

update SCHEMA_VERSION set VERSION=58;

-- We want the configs we insert to have incrementing CONFIG_ID values,
-- so we create an incrementing sequence here and use it to generate the
-- values for the CONFIG_ID column. We drop the sequence after the inserts.
create sequence CONFIG_ID_SEQ;

-- We set the initial value to be the max of the CONFIG_ID column so that
-- the first row we insert will have an ID of max(CONFIG_ID) + 1.
select setval('CONFIG_ID_SEQ', max(CONFIG_ID)) from CONFIGS;

-- Now we modify the column to use the CONFIG_ID_SEQ sequence for default
-- values.
alter table CONFIGS
    alter column CONFIG_ID set default nextval('CONFIG_ID_SEQ');

-- Now we copy over any role instance-level configs. This cannot be refactored
-- with the role type-level copying below using an OR because we cannot have
-- multiple rows with the same ATTR, SERVICE_ID, and REVISION_ID.
-- The "not exists" at the end is included to prevent exactly that problem.

insert into CONFIGS(ATTR, VALUE, SERVICE_ID, REVISION_ID)
select C1.ATTR, C1.VALUE, MR.SERVICE_ID, MR.CONFIG_REVISION_ID
from CONFIGS C1 cross join SERVICES MR
where C1.REVISION_ID=(
    select MGMT.CONFIG_REVISION_ID from ROLES AMON, SERVICES MGMT
    where AMON.SERVICE_ID = MGMT.SERVICE_ID and
          AMON.ROLE_TYPE = 'ACTIVITYMONITOR')
and (C1.ROLE_ID=(select ROLE_ID from ROLES where ROLE_TYPE='ACTIVITYMONITOR'))
and MR.SERVICE_TYPE='MAPREDUCE'
and C1.ATTR in (
    'firehose_activity_duration_rules',
    'firehose_activity_slow_alert',
    'firehose_activity_failure_alert')
and not exists(
    select ATTR, SERVICE_ID, REVISION_ID from CONFIGS C2
    where C2.ATTR = C1.ATTR and
          C2.SERVICE_ID = MR.SERVICE_ID and
          C2.REVISION_ID = MR.CONFIG_REVISION_ID);

-- Now we copy over any role type-level configs. This cannot be refactored
-- with the role instance-level copying above (see above for details).

insert into CONFIGS(ATTR, VALUE, SERVICE_ID, REVISION_ID)
select C1.ATTR, C1.VALUE, MR.SERVICE_ID, MR.CONFIG_REVISION_ID
from CONFIGS C1 cross join SERVICES MR
where C1.REVISION_ID=(
    select MGMT.CONFIG_REVISION_ID from ROLES AMON, SERVICES MGMT
    where AMON.SERVICE_ID = MGMT.SERVICE_ID and
          AMON.ROLE_TYPE = 'ACTIVITYMONITOR')
and C1.ROLE_TYPE='ACTIVITYMONITOR'
and MR.SERVICE_TYPE='MAPREDUCE'
and C1.ATTR in (
    'firehose_activity_duration_rules',
    'firehose_activity_slow_alert',
    'firehose_activity_failure_alert')
and not exists(
    select ATTR, SERVICE_ID, REVISION_ID from CONFIGS C2
    where C2.ATTR = C1.ATTR and
          C2.SERVICE_ID = MR.SERVICE_ID and
          C2.REVISION_ID = MR.CONFIG_REVISION_ID);

-- Now we remove the default value reference from the CONFIG_ID column.
alter table CONFIGS
    alter column CONFIG_ID drop default;

-- Now that we're done with everything, let's drop the sequence.
drop sequence CONFIG_ID_SEQ;
